@@ -34,6 +34,7 @@
  * blk.c -- block memory pool entry points for libpmem
  */
 
+#include <inttypes.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -42,7 +43,6 @@
 #include <errno.h>
 #include <time.h>
 #include <stdint.h>
-#include <pthread.h>
 #include <endian.h>
 
 #include "libpmem.h"
@@ -53,9 +53,10 @@
 #include "out.h"
 #include "btt.h"
 #include "blk.h"
+#include "util.h"
 #include "sys_util.h"
+#include "util_pmem.h"
 #include "valgrind_internal.h"
-
 /*
  * lane_enter -- (internal) acquire a unique lane number
  */
@@ -92,11 +93,11 @@ nsread(void *ns, unsigned lane, void *buf, size_t count, uint64_t off)
 {
 	struct pmemblk *pbp = (struct pmemblk *)ns;
 
-	LOG(13, "pbp %p lane %u count %zu off %ju", pbp, lane, count, off);
+	LOG(13, "pbp %p lane %u count %zu off %" PRIu64, pbp, lane, count, off);
 
 	if (off + count > pbp->datasize) {
 		ERR("offset + count (%zu) past end of data area (%zu)",
-				off + count, pbp->datasize);
+				(size_t)off + count, pbp->datasize);
 		errno = EINVAL;
 		return -1;
 	}
@@ -118,11 +119,11 @@ nswrite(void *ns, unsigned lane, const void *buf, size_t count,
 {
 	struct pmemblk *pbp = (struct pmemblk *)ns;
 
-	LOG(13, "pbp %p lane %u count %zu off %ju", pbp, lane, count, off);
+	LOG(13, "pbp %p lane %u count %zu off %" PRIu64, pbp, lane, count, off);
 
 	if (off + count > pbp->datasize) {
 		ERR("offset + count (%zu) past end of data area (%zu)",
-				off + count, pbp->datasize);
+				(size_t)off + count, pbp->datasize);
 		errno = EINVAL;
 		return -1;
 	}
@@ -173,13 +174,13 @@ nsmap(void *ns, unsigned lane, void **addrp, size_t len, uint64_t off)
 {
 	struct pmemblk *pbp = (struct pmemblk *)ns;
 
-	LOG(12, "pbp %p lane %u len %zu off %ju", pbp, lane, len, off);
+	LOG(12, "pbp %p lane %u len %zu off %" PRIu64, pbp, lane, len, off);
 
 	ASSERT(((ssize_t)len) >= 0);
 
 	if (off + len >= pbp->datasize) {
 		ERR("offset + len (%zu) past end of data area (%zu)",
-				off + len, pbp->datasize - 1);
+				(size_t)off + len, pbp->datasize - 1);
 		errno = EINVAL;
 		return -1;
 	}
@@ -230,11 +231,11 @@ nszero(void *ns, unsigned lane, size_t count, uint64_t off)
 {
 	struct pmemblk *pbp = (struct pmemblk *)ns;
 
-	LOG(13, "pbp %p lane %u count %zu off %ju", pbp, lane, count, off);
+	LOG(13, "pbp %p lane %u count %zu off %" PRIu64, pbp, lane, count, off);
 
 	if (off + count > pbp->datasize) {
 		ERR("offset + count (%zu) past end of data area (%zu)",
-				off + count, pbp->datasize);
+				(size_t)off + count, pbp->datasize);
 		errno = EINVAL;
 		return -1;
 	}
@@ -263,28 +264,26 @@ static struct ns_callback ns_cb = {
 };
 
 /*
- * pmemblk_descr_create -- (internal) create block memory pool descriptor
+ * blk_descr_create -- (internal) create block memory pool descriptor
  */
-static int
-pmemblk_descr_create(PMEMblkpool *pbp, uint32_t bsize, int zeroed)
+static void
+blk_descr_create(PMEMblkpool *pbp, uint32_t bsize, int zeroed)
 {
 	LOG(3, "pbp %p bsize %u zeroed %d", pbp, bsize, zeroed);
 
 	/* create the required metadata */
 	pbp->bsize = htole32(bsize);
-	PERSIST_GENERIC(pbp->is_pmem, &pbp->bsize, sizeof(bsize));
+	util_persist(pbp->is_pmem, &pbp->bsize, sizeof(bsize));
 
 	pbp->is_zeroed = zeroed;
-	PERSIST_GENERIC(pbp->is_pmem, &pbp->is_zeroed, sizeof(pbp->is_zeroed));
-
-	return 0;
+	util_persist(pbp->is_pmem, &pbp->is_zeroed, sizeof(pbp->is_zeroed));
 }
 
 /*
- * pmemblk_descr_check -- (internal) validate block memory pool descriptor
+ * blk_descr_check -- (internal) validate block memory pool descriptor
  */
 static int
-pmemblk_descr_check(PMEMblkpool *pbp, size_t *bsize)
+blk_descr_check(PMEMblkpool *pbp, size_t *bsize)
 {
 	LOG(3, "pbp %p bsize %zu", pbp, *bsize);
 
@@ -302,10 +301,10 @@ pmemblk_descr_check(PMEMblkpool *pbp, size_t *bsize)
 }
 
 /*
- * pmemblk_runtime_init -- (internal) initialize block memory pool runtime data
+ * blk_runtime_init -- (internal) initialize block memory pool runtime data
  */
 static int
-pmemblk_runtime_init(PMEMblkpool *pbp, size_t bsize, int rdonly)
+blk_runtime_init(PMEMblkpool *pbp, size_t bsize, int rdonly)
 {
 	LOG(3, "pbp %p bsize %zu rdonly %d",
 			pbp, bsize, rdonly);
@@ -340,7 +339,7 @@ pmemblk_runtime_init(PMEMblkpool *pbp, size_t bsize, int rdonly)
 
 	/* things free by "goto err" if not NULL */
 	struct btt *bttp = NULL;
-	pthread_mutex_t *locks = NULL;
+	os_mutex_t *locks = NULL;
 
 	bttp = btt_init(pbp->datasize, (uint32_t)bsize, pbp->hdr.poolset_uuid,
 			(unsigned)ncpus * 2, pbp, &ns_cb);
@@ -358,13 +357,13 @@ pmemblk_runtime_init(PMEMblkpool *pbp, size_t bsize, int rdonly)
 	}
 
 	for (unsigned i = 0; i < pbp->nlane; i++)
-		util_mutex_init(&locks[i], NULL);
+		util_mutex_init(&locks[i]);
 
 	pbp->locks = locks;
 
 #ifdef DEBUG
 	/* initialize debug lock */
-	util_mutex_init(&pbp->write_lock, NULL);
+	util_mutex_init(&pbp->write_lock);
 #endif
 
 	/*
@@ -390,11 +389,13 @@ err:
 }
 
 /*
- * pmemblk_create -- create a block memory pool
+ * pmemblk_createU -- create a block memory pool
  */
+#ifndef _WIN32
+static inline
+#endif
 PMEMblkpool *
-pmemblk_create(const char *path, size_t bsize, size_t poolsize,
-		mode_t mode)
+pmemblk_createU(const char *path, size_t bsize, size_t poolsize, mode_t mode)
 {
 	LOG(3, "path %s bsize %zu poolsize %zu mode %o",
 			path, bsize, poolsize, mode);
@@ -442,13 +443,10 @@ pmemblk_create(const char *path, size_t bsize, size_t poolsize,
 	ASSERT(!pbp->is_dev_dax || pbp->is_pmem);
 
 	/* create pool descriptor */
-	if (pmemblk_descr_create(pbp, (uint32_t)bsize, set->zeroed) != 0) {
-		LOG(2, "descriptor creation failed");
-		goto err;
-	}
+	blk_descr_create(pbp, (uint32_t)bsize, set->zeroed);
 
 	/* initialize runtime parts */
-	if (pmemblk_runtime_init(pbp, bsize, 0) != 0) {
+	if (blk_runtime_init(pbp, bsize, 0) != 0) {
 		ERR("pool initialization failed");
 		goto err;
 	}
@@ -464,14 +462,41 @@ pmemblk_create(const char *path, size_t bsize, size_t poolsize,
 err:
 	LOG(4, "error clean up");
 	int oerrno = errno;
-	util_poolset_close(set, 1);
+	util_poolset_close(set, DELETE_CREATED_PARTS);
 	errno = oerrno;
 	return NULL;
 }
 
+#ifndef _WIN32
+/*
+ * pmemblk_create -- create a block memory pool
+ */
+PMEMblkpool *
+pmemblk_create(const char *path, size_t bsize, size_t poolsize, mode_t mode)
+{
+	return pmemblk_createU(path, bsize, poolsize, mode);
+}
+#else
+/*
+ * pmemblk_createW -- create a block memory pool
+ */
+PMEMblkpool *
+pmemblk_createW(const wchar_t *path, size_t bsize, size_t poolsize,
+	mode_t mode)
+{
+	char *upath = util_toUTF8(path);
+	if (upath == NULL)
+		return NULL;
+
+	PMEMblkpool *ret = pmemblk_createU(upath, bsize, poolsize, mode);
+
+	util_free_UTF8(upath);
+	return ret;
+}
+#endif
 
 /*
- * pmemblk_open_common -- (internal) open a block memory pool
+ * blk_open_common -- (internal) open a block memory pool
  *
  * This routine does all the work, but takes a cow flag so internal
  * calls can map a read-only pool if required.
@@ -480,7 +505,7 @@ err:
  * will supply the block size).
  */
 static PMEMblkpool *
-pmemblk_open_common(const char *path, size_t bsize, int cow)
+blk_open_common(const char *path, size_t bsize, int cow)
 {
 	LOG(3, "path %s bsize %zu cow %d", path, bsize, cow);
 
@@ -519,13 +544,13 @@ pmemblk_open_common(const char *path, size_t bsize, int cow)
 	}
 
 	/* validate pool descriptor */
-	if (pmemblk_descr_check(pbp, &bsize) != 0) {
+	if (blk_descr_check(pbp, &bsize) != 0) {
 		LOG(2, "descriptor check failed");
 		goto err;
 	}
 
 	/* initialize runtime parts */
-	if (pmemblk_runtime_init(pbp, bsize, set->rdonly) != 0) {
+	if (blk_runtime_init(pbp, bsize, set->rdonly) != 0) {
 		ERR("pool initialization failed");
 		goto err;
 	}
@@ -538,21 +563,52 @@ pmemblk_open_common(const char *path, size_t bsize, int cow)
 err:
 	LOG(4, "error clean up");
 	int oerrno = errno;
-	util_poolset_close(set, 0);
+	util_poolset_close(set, DO_NOT_DELETE_PARTS);
 	errno = oerrno;
 	return NULL;
 }
 
+/*
+ * pmemblk_openU -- open a block memory pool
+ */
+#ifndef _WIN32
+static inline
+#endif
+PMEMblkpool *
+pmemblk_openU(const char *path, size_t bsize)
+{
+	LOG(3, "path %s bsize %zu", path, bsize);
+
+	return blk_open_common(path, bsize, 0);
+}
+
+#ifndef _WIN32
 /*
  * pmemblk_open -- open a block memory pool
  */
 PMEMblkpool *
 pmemblk_open(const char *path, size_t bsize)
 {
-	LOG(3, "path %s bsize %zu", path, bsize);
-
-	return pmemblk_open_common(path, bsize, 0);
+	return pmemblk_openU(path, bsize);
 }
+#else
+/*
+ * pmemblk_openW -- open a block memory pool
+ */
+PMEMblkpool *
+pmemblk_openW(const wchar_t *path, size_t bsize)
+{
+	char *upath = util_toUTF8(path);
+	if (upath == NULL)
+		return NULL;
+
+	PMEMblkpool *ret = pmemblk_openU(upath, bsize);
+
+	util_free_UTF8(upath);
+	return ret;
+}
+#endif
+
 
 /*
  * pmemblk_close -- close a block memory pool
@@ -565,16 +621,16 @@ pmemblk_close(PMEMblkpool *pbp)
 	btt_fini(pbp->bttp);
 	if (pbp->locks) {
 		for (unsigned i = 0; i < pbp->nlane; i++)
-			pthread_mutex_destroy(&pbp->locks[i]);
+			os_mutex_destroy(&pbp->locks[i]);
 		Free((void *)pbp->locks);
 	}
 
 #ifdef DEBUG
 	/* destroy debug lock */
-	pthread_mutex_destroy(&pbp->write_lock);
+	os_mutex_destroy(&pbp->write_lock);
 #endif
 
-	util_poolset_close(pbp->set, 0);
+	util_poolset_close(pbp->set, DO_NOT_DELETE_PARTS);
 }
 
 /*
@@ -718,17 +774,20 @@ pmemblk_set_error(PMEMblkpool *pbp, long long blockno)
 }
 
 /*
- * pmemblk_check -- block memory pool consistency check
+ * pmemblk_checkU -- block memory pool consistency check
  */
+#ifndef _WIN32
+static inline
+#endif
 int
-pmemblk_check(const char *path, size_t bsize)
+pmemblk_checkU(const char *path, size_t bsize)
 {
 	LOG(3, "path \"%s\" bsize %zu", path, bsize);
 
 	/* map the pool read-only */
-	PMEMblkpool *pbp = pmemblk_open_common(path, bsize, 1);
+	PMEMblkpool *pbp = blk_open_common(path, bsize, 1);
 	if (pbp == NULL)
-		return -1;	/* errno set by pmemblk_open_common() */
+		return -1;	/* errno set by blk_open_common() */
 
 	int retval = btt_check(pbp->bttp);
 	int oerrno = errno;
@@ -738,6 +797,33 @@ pmemblk_check(const char *path, size_t bsize)
 	return retval;
 }
 
+#ifndef _WIN32
+/*
+ * pmemblk_check -- block memory pool consistency check
+ */
+int
+pmemblk_check(const char *path, size_t bsize)
+{
+	return pmemblk_checkU(path, bsize);
+}
+#else
+/*
+ * pmemblk_checkW -- block memory pool consistency check
+ */
+int
+pmemblk_checkW(const wchar_t *path, size_t bsize)
+{
+	char *upath = util_toUTF8(path);
+	if (upath == NULL)
+		return -1;
+
+	int ret = pmemblk_checkU(upath, bsize);
+
+	util_free_UTF8(upath);
+	return ret;
+}
+#endif
+
 
 #ifdef _MSC_VER
 /*
