@@ -58,8 +58,11 @@
 #include "obj.h"
 #include "btt.h"
 #include "file.h"
+#include "os.h"
 #include "set.h"
 #include "check_util.h"
+#include "util_pmem.h"
+#include "mmap.h"
 
 /* arbitrary size of a maximum file part being read / write at once */
 #define RW_BUFFERING_SIZE (128 * 1024 * 1024)
@@ -71,7 +74,7 @@ static inline off_t
 pool_btt_lseek(struct pool_data *pool, off_t offset, int whence)
 {
 	off_t result;
-	if ((result = util_lseek(pool->set_file->fd, offset, whence)) == -1)
+	if ((result = os_lseek(pool->set_file->fd, offset, whence)) == -1)
 		ERR("!lseek");
 
 	return result;
@@ -172,7 +175,7 @@ pool_set_read_header(const char *fname, struct pool_hdr *hdr)
 	}
 
 err_close_part:
-	close(fdp);
+	os_close(fdp);
 
 err_pool_set:
 	util_poolset_free(set);
@@ -239,11 +242,11 @@ pool_params_from_header(struct pool_params *params, const struct pool_hdr *hdr)
 }
 
 /*
- * pmempool_check_type_to_pool_type -- (internal) convert check pool type to
+ * pool_check_type_to_pool_type -- (internal) convert check pool type to
  *	internal pool type value
  */
 static enum pool_type
-pmempool_check_type_to_pool_type(enum pmempool_pool_type check_pool_type)
+pool_check_type_to_pool_type(enum pmempool_pool_type check_pool_type)
 {
 	switch (check_pool_type) {
 	case PMEMPOOL_POOL_TYPE_LOG:
@@ -278,8 +281,8 @@ pool_params_parse(const PMEMpoolcheck *ppc, struct pool_params *params,
 
 	int ret = 0;
 
-	util_stat_t stat_buf;
-	ret = util_fstat(fd, &stat_buf);
+	os_stat_t stat_buf;
+	ret = os_fstat(fd, &stat_buf);
 	if (ret)
 		goto out_close;
 
@@ -294,7 +297,7 @@ pool_params_parse(const PMEMpoolcheck *ppc, struct pool_params *params,
 		 * Need to close the poolset because it will be opened with
 		 * flock in the following instructions.
 		 */
-		close(fd);
+		os_close(fd);
 		fd = -1;
 
 		if (check) {
@@ -348,9 +351,8 @@ pool_params_parse(const PMEMpoolcheck *ppc, struct pool_params *params,
 			goto out_close;
 		}
 		params->size = (size_t)s;
-		addr = mmap(NULL, (uint64_t)params->size, PROT_READ,
-			MAP_SHARED, fd, 0);
-		if (addr == MAP_FAILED) {
+		addr = util_map(fd, params->size, MAP_SHARED, 1, 0);
+		if (addr == NULL) {
 			ret = -1;
 			goto out_close;
 		}
@@ -371,7 +373,7 @@ pool_params_parse(const PMEMpoolcheck *ppc, struct pool_params *params,
 
 	if (ppc->args.pool_type != PMEMPOOL_POOL_TYPE_DETECT) {
 		enum pool_type declared_type =
-			pmempool_check_type_to_pool_type(ppc->args.pool_type);
+			pool_check_type_to_pool_type(ppc->args.pool_type);
 		if ((params->type & ~declared_type) != 0) {
 			ERR("declared pool type does not match");
 			ret = 1;
@@ -384,9 +386,8 @@ pool_params_parse(const PMEMpoolcheck *ppc, struct pool_params *params,
 		memcpy(&pbp, addr, sizeof(pbp));
 		params->blk.bsize = le32toh(pbp.bsize);
 	} else if (params->type == POOL_TYPE_OBJ) {
-		struct pmemobjpool pop;
-		memcpy(&pop, addr, sizeof(pop));
-		memcpy(params->obj.layout, pop.layout,
+		struct pmemobjpool *pop = addr;
+		memcpy(params->obj.layout, pop->layout,
 			PMEMOBJ_MAX_LAYOUT);
 	}
 
@@ -394,7 +395,7 @@ out_unmap:
 	if (params->is_poolset) {
 		ASSERTeq(fd, -1);
 		ASSERTne(addr, NULL);
-		util_poolset_close(set, 0);
+		util_poolset_close(set, DO_NOT_DELETE_PARTS);
 	} else if (!is_btt) {
 		ASSERTne(fd, -1);
 		ASSERTne(addr, NULL);
@@ -402,7 +403,7 @@ out_unmap:
 	}
 out_close:
 	if (fd != -1)
-		close(fd);
+		os_close(fd);
 	return ret;
 }
 
@@ -444,8 +445,8 @@ pool_set_file_open(const char *fname, struct pool_params *params, int rdonly)
 		file->size = params->size;
 	}
 
-	util_stat_t buf;
-	if (util_stat(path, &buf)) {
+	os_stat_t buf;
+	if (os_stat(path, &buf)) {
 		ERR("%s", path);
 		goto err_close_poolset;
 	}
@@ -456,9 +457,9 @@ pool_set_file_open(const char *fname, struct pool_params *params, int rdonly)
 
 err_close_poolset:
 	if (params->type != POOL_TYPE_BTT)
-		util_poolset_close(file->poolset, 0);
-	else
-		close(file->fd);
+		util_poolset_close(file->poolset, DO_NOT_DELETE_PARTS);
+	else if (file->fd != -1)
+		os_close(file->fd);
 err_free_fname:
 	free(file->fname);
 err:
@@ -474,7 +475,7 @@ pool_set_parse(struct pool_set **setp, const char *path)
 {
 	LOG(3, "setp %p path %s", setp, path);
 
-	int fd = open(path, O_RDONLY);
+	int fd = os_open(path, O_RDONLY);
 	int ret = 0;
 
 	if (fd < 0)
@@ -486,7 +487,7 @@ pool_set_parse(struct pool_set **setp, const char *path)
 	}
 
 err_close:
-	close(fd);
+	os_close(fd);
 	return ret;
 }
 
@@ -559,12 +560,12 @@ pool_set_file_close(struct pool_set_file *file)
 	LOG(3, NULL);
 
 	if (file->poolset)
-		util_poolset_close(file->poolset, 0);
+		util_poolset_close(file->poolset, DO_NOT_DELETE_PARTS);
 	else if (file->addr) {
 		munmap(file->addr, file->size);
-		close(file->fd);
+		os_close(file->fd);
 	} else if (file->fd)
-		close(file->fd);
+		os_close(file->fd);
 
 	free(file->fname);
 	free(file);
@@ -612,6 +613,9 @@ pool_set_file_map(struct pool_set_file *file, uint64_t offset)
 
 /*
  * pool_read -- read from pool set file or regular file
+ *
+ * 'buff' has to be a buffer at least 'nbytes' long
+ * 'off' is an offset from the beginning of the pool
  */
 int
 pool_read(struct pool_data *pool, void *buff, size_t nbytes, uint64_t off)
@@ -633,6 +637,9 @@ pool_read(struct pool_data *pool, void *buff, size_t nbytes, uint64_t off)
 
 /*
  * pool_write -- write to pool set file or regular file
+ *
+ * 'buff' has to be a buffer at least 'nbytes' long
+ * 'off' is an offset from the beginning of the pool
  */
 int
 pool_write(struct pool_data *pool, const void *buff, size_t nbytes,
@@ -641,9 +648,11 @@ pool_write(struct pool_data *pool, const void *buff, size_t nbytes,
 	if (off + nbytes > pool->set_file->size)
 		return -1;
 
-	if (pool->params.type != POOL_TYPE_BTT)
+	if (pool->params.type != POOL_TYPE_BTT) {
 		memcpy((char *)pool->set_file->addr + off, buff, nbytes);
-	else {
+		util_persist_auto(pool->params.is_dev_dax,
+				(char *)pool->set_file->addr + off, nbytes);
+	} else {
 		if (pool_btt_lseek(pool, (off_t)off, SEEK_SET) == -1)
 			return -1;
 		if ((size_t)pool_btt_write(pool, buff, nbytes) != nbytes)
@@ -661,7 +670,7 @@ pool_copy(struct pool_data *pool, const char *dst_path, int overwrite)
 {
 	struct pool_set_file *file = pool->set_file;
 	int dfd;
-	if (!access(dst_path, F_OK)) {
+	if (!os_access(dst_path, F_OK)) {
 		if (!overwrite) {
 			errno = EEXIST;
 			return -1;
@@ -679,8 +688,8 @@ pool_copy(struct pool_data *pool, const char *dst_path, int overwrite)
 		return -1;
 
 	int result = 0;
-	util_stat_t stat_buf;
-	if (util_stat(file->fname, &stat_buf)) {
+	os_stat_t stat_buf;
+	if (os_stat(file->fname, &stat_buf)) {
 		result = -1;
 		goto out_close;
 	}
@@ -710,7 +719,10 @@ pool_copy(struct pool_data *pool, const char *dst_path, int overwrite)
 		goto out_unmap;
 	}
 
-	pool_btt_lseek(pool, 0, SEEK_SET);
+	if (pool_btt_lseek(pool, 0, SEEK_SET) == -1) {
+		result = -1;
+		goto out_free;
+	}
 	ssize_t buf_read = 0;
 	void *dst = daddr;
 	while ((buf_read = pool_btt_read(pool, buf, RW_BUFFERING_SIZE))) {
@@ -721,12 +733,12 @@ pool_copy(struct pool_data *pool, const char *dst_path, int overwrite)
 		dst  = (void *)((ssize_t)dst + buf_read);
 	}
 
+out_free:
 	free(buf);
 out_unmap:
 	munmap(daddr, file->size);
 out_close:
-	if (dfd >= 0)
-		close(dfd);
+	(void) os_close(dfd);
 	return result;
 }
 
@@ -741,8 +753,8 @@ pool_set_part_copy(struct pool_set_part *dpart, struct pool_set_part *spart,
 
 	int result = 0;
 
-	util_stat_t stat_buf;
-	if (util_stat(spart->path, &stat_buf)) {
+	os_stat_t stat_buf;
+	if (os_fstat(spart->fd, &stat_buf)) {
 		ERR("!util_stat");
 		return -1;
 	}
@@ -756,7 +768,7 @@ pool_set_part_copy(struct pool_set_part *dpart, struct pool_set_part *spart,
 	int is_pmem;
 	void *daddr;
 
-	if (!access(dpart->path, F_OK)) {
+	if (!os_access(dpart->path, F_OK)) {
 		if (!overwrite) {
 			errno = EEXIST;
 			result = -1;
@@ -781,11 +793,13 @@ pool_set_part_copy(struct pool_set_part *dpart, struct pool_set_part *spart,
 		goto out_sunmap;
 	}
 
+	ASSERT(dmapped >= smapped);
+
 	if (is_pmem) {
 		pmem_memcpy_persist(daddr, saddr, smapped);
 	} else {
 		memcpy(daddr, saddr, smapped);
-		PERSIST_GENERIC(dpart->is_dev_dax, daddr, smapped);
+		pmem_msync(daddr, smapped);
 	}
 
 	pmem_unmap(daddr, dmapped);
@@ -805,7 +819,9 @@ pool_memset(struct pool_data *pool, uint64_t off, int c, size_t count)
 	if (pool->params.type != POOL_TYPE_BTT)
 		memset((char *)off, 0, count);
 	else {
-		pool_btt_lseek(pool, (off_t)off, SEEK_SET);
+		if (pool_btt_lseek(pool, (off_t)off, SEEK_SET) == -1)
+			return -1;
+
 		size_t zero_size = min(count, RW_BUFFERING_SIZE);
 		void *buf = malloc(zero_size);
 		if (!buf) {
