@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2016, Intel Corporation
+ * Copyright 2014-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -76,14 +76,14 @@ main(int argc, char *argv[])
 	for (int arg = 1; arg < argc; arg++) {
 		int fd = OPEN(argv[arg], O_RDONLY);
 
-		ut_util_stat_t stbuf;
+		os_stat_t stbuf;
 		FSTAT(fd, &stbuf);
 
 		void *addr =
-			MMAP(0, stbuf.st_size, PROT_READ|PROT_WRITE,
+			MMAP(NULL, stbuf.st_size, PROT_READ|PROT_WRITE,
 					MAP_PRIVATE, fd, 0);
 
-		close(fd);
+		CLOSE(fd);
 
 		uint64_t *ptr = addr;
 
@@ -136,6 +136,8 @@ main(int argc, char *argv[])
 
 			ptr++;
 		}
+
+		MUNMAP(addr, stbuf.st_size);
 	}
 
 	DONE(NULL);
