@@ -1,5 +1,5 @@
 /*
- * Copyright 2016, Intel Corporation
+ * Copyright 2016-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,9 +36,9 @@
 
 #include "unittest.h"
 
-#include <libpmemobj/p.hpp>
-#include <libpmemobj/persistent_ptr.hpp>
-#include <libpmemobj/pool.hpp>
+#include <libpmemobj++/p.hpp>
+#include <libpmemobj++/persistent_ptr.hpp>
+#include <libpmemobj++/pool.hpp>
 
 namespace nvobj = nvml::obj;
 
@@ -68,7 +68,7 @@ pool_create(const char *path, const char *layout, size_t poolsize,
 		return;
 	}
 
-	struct stat stbuf;
+	os_stat_t stbuf;
 	STAT(path, &stbuf);
 
 	UT_OUT("%s: file size %zu mode 0%o", path, stbuf.st_size,
@@ -173,9 +173,9 @@ main(int argc, char *argv[])
 
 	switch (argv[1][0]) {
 		case 'c':
-			poolsize = strtoul(argv[4], NULL, 0) *
+			poolsize = strtoul(argv[4], nullptr, 0) *
 				MB; /* in megabytes */
-			mode = strtoul(argv[5], NULL, 8);
+			mode = strtoul(argv[5], nullptr, 8);
 
 			pool_create(argv[2], layout, poolsize, mode);
 			break;
@@ -183,9 +183,9 @@ main(int argc, char *argv[])
 			pool_open(argv[2], layout);
 			break;
 		case 'd':
-			poolsize = strtoul(argv[4], NULL, 0) *
+			poolsize = strtoul(argv[4], nullptr, 0) *
 				MB; /* in megabytes */
-			mode = strtoul(argv[5], NULL, 8);
+			mode = strtoul(argv[5], nullptr, 8);
 
 			double_close(argv[2], layout, poolsize, mode);
 			break;
@@ -196,5 +196,5 @@ main(int argc, char *argv[])
 			UT_FATAL("unknown operation");
 	}
 
-	DONE(NULL);
+	DONE(nullptr);
 }
