@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2016, Intel Corporation
+ * Copyright 2014-2017, Intel Corporation
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -51,6 +51,7 @@
 #include <err.h>
 #include <assert.h>
 #include <endian.h>
+#include <libpmem.h>
 #include "common.h"
 #include "output.h"
 #include "btt.h"
@@ -295,12 +296,24 @@ static const char *help_str =
  * long_options -- command line options
  */
 static const struct option long_options[] = {
-	{"verbose",	no_argument,		0,	'v'},
-	{"help",	no_argument,		0,	'?'},
-	{"replica",	required_argument,	0,	'r'},
-	{0,		0,			0,	 0 },
+	{"verbose",	no_argument,		NULL,	'v'},
+	{"help",	no_argument,		NULL,	'?'},
+	{"replica",	required_argument,	NULL,	'r'},
+	{NULL,		0,			NULL,	 0 },
 };
 
+/*
+ * pmemspoil_persist -- flush data to persistence
+ */
+static void
+pmemspoil_persist(void *addr, size_t size)
+{
+	if (pmem_is_pmem(addr, size))
+		pmem_persist(addr, size);
+	else
+		pmem_msync(addr, size);
+}
+
 /*
  * print_usage -- print application usage short description
  */
@@ -587,6 +600,8 @@ pmemspoil_process_char(struct pmemspoil *psp, struct pmemspoil_list *pfp,
 	len = min(len, strlen(pfp->value));
 	memcpy(str, pfp->value, len);
 
+	pmemspoil_persist(str, len);
+
 	return 0;
 }
 
@@ -606,6 +621,8 @@ pmemspoil_process_uint16_t(struct pmemspoil *psp, struct pmemspoil_list *pfp,
 	else
 		*valp = v;
 
+	pmemspoil_persist(valp, sizeof(*valp));
+
 	return 0;
 }
 
@@ -625,6 +642,8 @@ pmemspoil_process_uint32_t(struct pmemspoil *psp, struct pmemspoil_list *pfp,
 	else
 		*valp = v;
 
+	pmemspoil_persist(valp, sizeof(*valp));
+
 	return 0;
 }
 
@@ -644,6 +663,8 @@ pmemspoil_process_uint64_t(struct pmemspoil *psp, struct pmemspoil_list *pfp,
 	else
 		*valp = v;
 
+	pmemspoil_persist(valp, sizeof(*valp));
+
 	return 0;
 }
 
@@ -1278,6 +1299,18 @@ pmemspoil_process(struct pmemspoil *psp,
 int
 main(int argc, char *argv[])
 {
+#ifdef _WIN32
+	wchar_t **wargv = CommandLineToArgvW(GetCommandLineW(), &argc);
+	for (int i = 0; i < argc; i++) {
+		argv[i] = util_toUTF8(wargv[i]);
+		if (argv[i] == NULL) {
+			for (i--; i >= 0; i--)
+				free(argv[i]);
+			outv_err("Error during arguments conversion\n");
+			return 1;
+		}
+	}
+#endif
 	char *appname = basename(argv[0]);
 	util_init();
 	int ret = 0;
@@ -1306,11 +1339,11 @@ main(int argc, char *argv[])
 		err(1, "%s", psp->fname);
 
 	if (pool_set_file_set_replica(psp->pfile, psp->replica)) {
-		outv_err("invalid replica argument max is %lu\n",
+		outv_err("invalid replica argument max is %u\n",
 				psp->pfile->poolset ?
 				psp->pfile->poolset->nreplicas :
 				0);
-		return -1;
+		return 1;
 	}
 
 	psp->addr = pool_set_file_map(psp->pfile, 0);
@@ -1334,6 +1367,9 @@ error:
 	pool_set_file_close(psp->pfile);
 
 	free(psp);
-
+#ifdef _WIN32
+	for (int i = argc; i > 0; i--)
+		free(argv[i - 1]);
+#endif
 	return ret;
 }
