@@ -1,5 +1,5 @@
 #
-# Copyright 2014-2016, Intel Corporation
+# Copyright 2014-2017, Intel Corporation
 # Copyright (c) 2016, Microsoft Corporation. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -33,6 +33,7 @@
 
 # make sure we have a well defined locale for string operations here
 export LC_ALL="C"
+#export LC_ALL="en_US.UTF-8"
 
 . ../testconfig.sh
 
@@ -43,7 +44,7 @@ export LC_ALL="C"
 [ "$CHECK_TYPE" ] || export CHECK_TYPE=auto
 [ "$CHECK_POOL" ] || export CHECK_POOL=0
 [ "$VERBOSE" ] || export VERBOSE=0
-
+[ "$SUFFIX" ] || export SUFFIX="😘⠝⠧⠍⠇ɗNVMLӜ⥺🙋"
 
 TOOLS=../tools
 # Paths to some useful tools
@@ -56,6 +57,7 @@ TOOLS=../tools
 [ "$PMEMDETECT" ] || PMEMDETECT=$TOOLS/pmemdetect/pmemdetect.static-nondebug
 [ "$FIP" ] || FIP=$TOOLS/fip/fip
 [ "$DDMAP" ] || DDMAP=$TOOLS/ddmap/ddmap
+[ "$CMPMAP" ] || CMPMAP=$TOOLS/cmpmap/cmpmap
 
 # force globs to fail if they don't match
 shopt -s failglob
@@ -144,28 +146,28 @@ fi
 
 REAL_FS=$FS
 if [ "$DIR" ]; then
-	DIR=$DIR/$curtestdir$UNITTEST_NUM
+	DIR=$DIR/$curtestdir$UNITTEST_NUM$SUFFIX
 else
 	case "$FS"
 	in
 	pmem)
-		DIR=$PMEM_FS_DIR/$curtestdir$UNITTEST_NUM
+		DIR=$PMEM_FS_DIR/$DIRSUFFIX/$curtestdir$UNITTEST_NUM$SUFFIX
 		if [ "$PMEM_FS_DIR_FORCE_PMEM" = "1" ]; then
 			export PMEM_IS_PMEM_FORCE=1
 		fi
 		;;
 	non-pmem)
-		DIR=$NON_PMEM_FS_DIR/$curtestdir$UNITTEST_NUM
+		DIR=$NON_PMEM_FS_DIR/$DIRSUFFIX/$curtestdir$UNITTEST_NUM$SUFFIX
 		;;
 	any)
 		if [ "$PMEM_FS_DIR" != "" ]; then
-			DIR=$PMEM_FS_DIR/$curtestdir$UNITTEST_NUM
+			DIR=$PMEM_FS_DIR/$DIRSUFFIX/$curtestdir$UNITTEST_NUM$SUFFIX
 			REAL_FS=pmem
 			if [ "$PMEM_FS_DIR_FORCE_PMEM" = "1" ]; then
 				export PMEM_IS_PMEM_FORCE=1
 			fi
 		elif [ "$NON_PMEM_FS_DIR" != "" ]; then
-			DIR=$NON_PMEM_FS_DIR/$curtestdir$UNITTEST_NUM
+			DIR=$NON_PMEM_FS_DIR/$DIRSUFFIX/$curtestdir$UNITTEST_NUM$SUFFIX
 			REAL_FS=non-pmem
 		else
 			echo "$UNITTEST_NAME: fs-type=any and both env vars are empty" >&2
@@ -173,13 +175,13 @@ else
 		fi
 		;;
 	none)
-		DIR=/dev/null/not_existing_dir/$curtestdir$UNITTEST_NUM
+		DIR=/dev/null/not_existing_dir/$DIRSUFFIX/$curtestdir$UNITTEST_NUM$SUFFIX
 		;;
 	*)
 		[ "$UNITTEST_QUIET" ] || echo "$UNITTEST_NAME: SKIP fs-type $FS (not configured)"
 		exit 0
-                ;;
-        esac
+		;;
+	esac
 fi
 
 if [ -d "$PMEM_FS_DIR" ]; then
@@ -236,6 +238,41 @@ RPMEM_LOG_LEVEL"
 
 export CHECK_POOL_LOG_FILE=check_pool_${BUILD}_${UNITTEST_NUM}.log
 
+#
+# store_exit_on_error -- store on a stack a sign that reflects the current state
+#                        of the 'errexit' shell option
+#
+function store_exit_on_error() {
+	if [ "${-#*e}" != "$-" ]; then
+		estack+=-
+	else
+		estack+=+
+	fi
+}
+
+#
+# restore_exit_on_error -- restore the state of the 'errexit' shell option
+#
+function restore_exit_on_error() {
+	if [ -z $estack ]; then
+		echo "error: store_exit_on_error function has to be called first"
+		exit 1
+	fi
+
+	eval "set ${estack:${#estack}-1:1}e"
+	estack=${estack%?}
+}
+
+#
+# disable_exit_on_error -- store the state of the 'errexit' shell option and
+#                          disable it
+#
+function disable_exit_on_error() {
+	store_exit_on_error
+	set +e
+}
+
+
 #
 # get_files -- print list of files in the current directory matching the given regex to stdout
 #
@@ -245,9 +282,9 @@ export CHECK_POOL_LOG_FILE=check_pool_${BUILD}_${UNITTEST_NUM}.log
 # example, to list all *.log files in the current directory
 #	get_files ".*\.log"
 function get_files() {
-	set +e
+	disable_exit_on_error
 	ls -1 | grep -E "^$*$"
-	set -e
+	restore_exit_on_error
 }
 
 #
@@ -257,7 +294,7 @@ function get_files() {
 # `find`, which fails if any file disappears in the middle of the operation.
 #
 function get_executables() {
-	set +e
+	disable_exit_on_error
 	for c in *
 	do
 		local rights=$(stat -c "%a %F" "$c" 2>/dev/null)
@@ -271,7 +308,7 @@ function get_executables() {
 			echo "$c"
 		fi
 	done
-	set -e
+	restore_exit_on_error
 }
 
 #
@@ -395,7 +432,7 @@ function create_holey_file() {
 #   psize:ppath[:cmd[:fsize[:mode]]]
 #
 # where:
-#   psize - part size
+#   psize - part size or AUTO (only for DAX device)
 #   ppath - path
 #   cmd   - (optional) can be:
 #            x - do nothing (may be skipped if there's no 'fsize', 'mode')
@@ -414,10 +451,11 @@ function create_holey_file() {
 #
 # example:
 #   The following command define a pool set consisting of two parts: 16MB
-#   and 32MB, and the replica with only one part of 48MB.  The first part file
-#   is not created, the second is zeroed.  The only replica part is non-zeroed.
-#   Also, the last file is read-only and its size does not match the information
-#   from pool set file. The last line describes a remote replica.
+#   and 32MB, a local replica with only one part of 48MB and a remote replica.
+#   The first part file is not created, the second is zeroed.  The only replica
+#   part is non-zeroed. Also, the last file is read-only and its size
+#   does not match the information from pool set file. The last line describes
+#   a remote replica.
 #
 #	create_poolset ./pool.set 16M:testfile1 32M:testfile2:z \
 #				R 48M:testfile3:n:11M:0400 \
@@ -434,11 +472,13 @@ function create_poolset() {
 			shift 1
 
 			cmd=$1
-			fparms=(${cmd//:/ })
 			shift 1
 
-			node=${fparms[0]}
-			desc=${fparms[1]}
+			# extract last ":" separated segment as descriptor
+			# extract everything before last ":" as node address
+			# this extraction method is compatible with IPv6 and IPv4
+			node=${cmd%:*}
+			desc=${cmd##*:}
 
 			echo "REPLICA $node $desc" >> $psfile
 			continue
@@ -458,14 +498,16 @@ function create_poolset() {
 		fsize=${fparms[0]}
 		fpath=${fparms[1]}
 		cmd=${fparms[2]}
-		asize=${fparams[3]}
+		asize=${fparms[3]}
 		mode=${fparms[4]}
 
 		if [ ! $asize ]; then
 			asize=$fsize
 		fi
 
-		asize=$(convert_to_bytes $asize)
+		if [ "$asize" != "AUTO" ]; then
+			asize=$(convert_to_bytes $asize)
+		fi
 
 		case "$cmd"
 		in
@@ -543,7 +585,7 @@ function get_trace() {
 	if [ "$check_type" = "memcheck" -a "$MEMCHECK_DONT_CHECK_LEAKS" != "1" ]; then
 		opts="$opts --leak-check=full"
 	fi
-	opts="$opts --suppressions=../ld.supp"
+	opts="$opts --suppressions=../ld.supp --suppressions=../memcheck-libunwind.supp"
 	if [ "$node" -ne -1 ]; then
 		exe=${NODE_VALGRINDEXE[$node]}
 		opts="$opts"
@@ -586,6 +628,15 @@ function expect_normal_exit() {
 		N=-1
 	fi
 
+	if [ -n "$TRACE" ]; then
+		case "$1"
+		in
+		*_on_node*)
+			echo "$UNITTEST_NAME: SKIP: TRACE is not supported if test is executed on remote nodes"
+			exit 0
+		esac
+	fi
+
 	local trace=$(get_trace $_CHECK_TYPE $VALGRIND_LOG_FILE $N)
 
 	if [ "$MEMCHECK_DONT_CHECK_LEAKS" = "1" -a "$CHECK_TYPE" = "memcheck" ]; then
@@ -614,7 +665,7 @@ function expect_normal_exit() {
 	        esac
 	fi
 
-	set +e
+	disable_exit_on_error
 	eval $ECHO LD_LIBRARY_PATH=$TEST_LD_LIBRARY_PATH LD_PRELOAD=$TEST_LD_PRELOAD \
 		$trace $*
 	ret=$?
@@ -626,7 +677,7 @@ function expect_normal_exit() {
 			mv $VALGRIND_LOG_FILE $new_log_file
 		done
 	fi
-	set -e
+	restore_exit_on_error
 
 	if [ "$ret" -ne "0" ]; then
 		if [ "$ret" -gt "128" ]; then
@@ -652,10 +703,11 @@ function expect_normal_exit() {
 
 		# ignore Ctrl-C
 		if [ $ret != 130 ]; then
-			for f in $(get_files "node_.*\.log"); do
+			for f in $(get_files "node_.*${UNITTEST_NUM}\.log"); do
 				dump_last_n_lines $f
 			done
-			dump_last_n_lines out$UNITTEST_NUM.log
+
+			dump_last_n_lines trace$UNITTEST_NUM.log
 			dump_last_n_lines $PMEM_LOG_FILE
 			dump_last_n_lines $PMEMOBJ_LOG_FILE
 			dump_last_n_lines $PMEMLOG_LOG_FILE
@@ -695,11 +747,20 @@ function expect_normal_exit() {
 # expect_abnormal_exit -- run a given command, expect it to exit non-zero
 #
 function expect_abnormal_exit() {
-	set +e
+	if [ -n "$TRACE" ]; then
+		case "$1"
+		in
+		*_on_node*)
+			echo "$UNITTEST_NAME: SKIP: TRACE is not supported if test is executed on remote nodes"
+			exit 0
+		esac
+	fi
+
+	disable_exit_on_error
 	eval $ECHO ASAN_OPTIONS="detect_leaks=0 ${ASAN_OPTIONS}" LD_LIBRARY_PATH=$TEST_LD_LIBRARY_PATH LD_PRELOAD=$TEST_LD_PRELOAD \
 	$TRACE $*
 	ret=$?
-	set -e
+	restore_exit_on_error
 
 	if [ "$ret" -eq "0" ]; then
 		msg="succeeded"
@@ -826,7 +887,7 @@ function require_dev_dax_node() {
 			exit 0
 		fi
 		local device_dax_path=${NODE_DEVICE_DAX_PATH[$node]}
-		local cmd="run_command ssh $SSH_OPTS ${NODE[$node]} cd $DIR && LD_LIBRARY_PATH=$REMOTE_LD_LIBRARY_PATH ../pmemdetect -d"
+		local cmd="ssh $SSH_OPTS ${NODE[$node]} cd $DIR && LD_LIBRARY_PATH=$REMOTE_LD_LIBRARY_PATH ../pmemdetect -d"
 	else
 		local prefix="$UNITTEST_NAME: SKIP"
 		if [ ${#DEVICE_DAX_PATH[@]} -lt $min ]; then
@@ -834,19 +895,18 @@ function require_dev_dax_node() {
 			exit 0
 		fi
 		local device_dax_path=${DEVICE_DAX_PATH[@]}
-		local var_name="DEVICE_DAX_PATH"
 		local cmd="$PMEMDETECT -d"
 	fi
 
 	for path in ${device_dax_path[@]}
 	do
-		set +e
+		disable_exit_on_error
 		out=$($cmd $path 2>&1)
 		ret=$?
-		set -e
+		restore_exit_on_error
 
 		if [ "$ret" == "0" ]; then
-			return
+			continue
 		elif [ "$ret" == "1" ]; then
 			echo "$prefix $out"
 			exit 0
@@ -874,6 +934,50 @@ function require_dax_devices() {
 	require_dev_dax_node $1
 }
 
+#
+# require_dax_device_alignments -- only allow script to continue if
+#    the internal Device DAX alignments are as specified.
+# If necessary, it sorts DEVICE_DAX_PATH entries to match
+# the requested alignment order.
+#
+# usage: require_dax_device_alignments alignment1 [ alignment2 ... ]
+#
+function require_dax_device_alignments() {
+	local cnt=${#DEVICE_DAX_PATH[@]}
+	local j=0
+
+	for alignment in $*
+	do
+		for (( i=j; i<cnt; i++ ))
+		do
+			#echo "j=$j i=$i alignment=$alignment"
+			path=${DEVICE_DAX_PATH[$i]}
+
+			disable_exit_on_error
+			out=`$PMEMDETECT -a $alignment $path 2>&1`
+			ret=$?
+			restore_exit_on_error
+
+			if [ "$ret" == "0" ]; then
+				#echo "found $path alignment=$alignment"
+				if [ $i -ne $j ]; then
+					tmp=${DEVICE_DAX_PATH[$j]}
+					DEVICE_DAX_PATH[$j]=$path
+					DEVICE_DAX_PATH[$i]=$tmp
+				fi
+				break
+			fi
+		done
+
+		if [ $i -eq $cnt ]; then
+			echo "$UNITTEST_NAME: SKIP Cannot find Device DAX #$j with alignment $alignment"
+			exit 0
+		fi
+
+		j=$(( j + 1 ))
+	done
+}
+
 #
 # require_node_dax_device -- only allow script to continue if specified node
 # has defined device dax in testconfig.sh
@@ -890,7 +994,10 @@ function require_fs_type() {
 	req_fs_type=1
 	for type in $*
 	do
-		[ "$type" = "$FS" ] &&
+		# treat any as either pmem or non-pmem
+		[ "$type" = "$FS" ] ||
+			([ -n "${FORCE_FS:+x}" ] && [ "$type" = "any" ] &&
+			[ "$FS" != "none" ]) &&
 		case "$REAL_FS"
 		in
 		pmem)
@@ -920,8 +1027,20 @@ function require_build_type() {
 	exit 0
 }
 
+#
+# require_command -- only allow script to continue if specified command exists
+#
+function require_command() {
+	if ! command -pv $1 1>/dev/null
+	then
+		echo "$UNITTEST_NAME: SKIP: '$1' command required"
+		exit 0
+	fi
+}
+
 #
 # require_pkg -- only allow script to continue if specified package exists
+#    usage: require_pkg <package name> [<package minimal version>]
 #
 function require_pkg() {
 	if ! command -v pkg-config 1>/dev/null
@@ -930,9 +1049,16 @@ function require_pkg() {
 		exit 0
 	fi
 
-	if ! pkg-config $1
+	local COMMAND="pkg-config $1"
+	local MSG="$UNITTEST_NAME: SKIP '$1' package"
+	if [ "$#" -eq "2" ]; then
+		COMMAND="$COMMAND --atleast-version $2"
+		MSG="$MSG (version >= $2)"
+	fi
+	MSG="$MSG required"
+	if ! $COMMAND
 	then
-		echo "$UNITTEST_NAME: SKIP '$1' package required"
+		echo "$MSG"
 		exit 0
 	fi
 }
@@ -940,6 +1066,7 @@ function require_pkg() {
 #
 # require_node_pkg -- only allow script to continue if specified package exists
 # on specified node
+#    usage: require_node_pkg <node> <package name> [<package minimal version>]
 #
 function require_node_pkg() {
 	validate_node_number $1
@@ -955,14 +1082,20 @@ function require_node_pkg() {
 	fi
 
 	COMMAND="$COMMAND pkg-config $1"
+	MSG="$UNITTEST_NAME: SKIP NODE $N: '$1' package"
+	if [ "$#" -eq "2" ]; then
+		COMMAND="$COMMAND --atleast-version $2"
+		MSG="$MSG (version >= $2)"
+	fi
+	MSG="$MSG required"
 
-	set +e
+	disable_exit_on_error
 	run_command ssh $SSH_OPTS ${NODE[$N]} "$COMMAND" 2>&1
 	ret=$?
-	set -e
+	restore_exit_on_error
 
 	if [ "$ret" == 1 ]; then
-		echo "$UNITTEST_NAME: SKIP NODE $N: '$1' package required"
+		echo "$MSG"
 		exit 0
 	fi
 }
@@ -1005,6 +1138,10 @@ function configure_valgrind() {
 		fi
 		require_valgrind_tool $CHECK_TYPE $3
 	fi
+
+	if [ "$UT_VALGRIND_SKIP_PRINT_MISMATCHED" == 1 ]; then
+		export UT_SKIP_PRINT_MISMATCHED=1
+	fi
 }
 
 #
@@ -1013,10 +1150,10 @@ function configure_valgrind() {
 #
 function require_valgrind() {
 	require_no_asan
-	set +e
+	disable_exit_on_error
 	VALGRINDEXE=`which valgrind 2>/dev/null`
 	local ret=$?
-	set -e
+	restore_exit_on_error
 	if [ $ret -ne 0 ]; then
 		echo "$UNITTEST_NAME: SKIP valgrind package required"
 		exit 0
@@ -1024,10 +1161,10 @@ function require_valgrind() {
 	[ $NODES_MAX -lt 0 ] && return;
 	for N in $NODES_SEQ; do
 		if [ "${NODE_VALGRINDEXE[$N]}" = "" ]; then
-			set +e
+			disable_exit_on_error
 			NODE_VALGRINDEXE[$N]=$(ssh $SSH_OPTS ${NODE[$N]} "which valgrind 2>/dev/null")
 			ret=$?
-			set -e
+			restore_exit_on_error
 			if [ $ret -ne 0 ]; then
 				echo "$UNITTEST_NAME: SKIP valgrind package required on remote node #$N"
 				exit 0
@@ -1050,6 +1187,10 @@ function require_valgrind_tool() {
 	[ -d "$2" ] && dir="$2" && binary=
 	pushd "$dir" > /dev/null
 	[ -n "$binary" ] || binary=$(get_executables)
+	if [ -z "$binary" ]; then
+		echo "require_valgrind_tool: error: no binary found" >&2
+		exit 1
+	fi
 	strings ${binary} 2>&1 | \
 	grep -q "compiled with support for Valgrind $tool" && true
 	if [ $? -ne 0 ]; then
@@ -1102,69 +1243,46 @@ function set_valgrind_exe_name() {
 	done
 }
 
-#
-# require_valgrind_dev_3_7 -- continue script execution only if
-#	version 3.7 (or later) of valgrind-devel package is installed
-#
-function require_valgrind_dev_3_7() {
-	require_valgrind
-	echo "
-        #include <valgrind/valgrind.h>
-        #if defined (VALGRIND_RESIZEINPLACE_BLOCK)
-        VALGRIND_VERSION_3_7_OR_LATER
-        #endif" | gcc ${EXTRA_CFLAGS} -E - 2>&1 | \
-		grep -q "VALGRIND_VERSION_3_7_OR_LATER" && return
-	echo "$UNITTEST_NAME: SKIP valgrind-devel package (ver 3.7 or later) required"
-	exit 0
-}
-
 #
 # valgrind_version -- returns Valgrind version
 #
 function valgrind_version() {
-	echo "#include <valgrind/valgrind.h>
-#if defined (__VALGRIND_MAJOR__) && defined (__VALGRIND_MINOR__)
-__VALGRIND_MAJOR__*100+__VALGRIND_MINOR__
-#else
-0
-#endif" | gcc ${EXTRA_CFLAGS} -E - | tail -n 1 | bc
+	require_valgrind
+	$VALGRINDEXE --version | sed "s/valgrind-\([0-9]*\)\.\([0-9]*\).*/\1*100+\2/" | bc
 }
 
 #
-# require_valgrind_dev_3_8 -- continue script execution only if
-#	version 3.8 (or later) of valgrind-devel package is installed
+# require_valgrind_dev_version -- continue script execution only if
+#	certain version (or later) of valgrind-devel package is installed
 #
-function require_valgrind_dev_3_8() {
+function require_valgrind_dev_version() {
 	require_valgrind
-	echo "
-        #include <valgrind/valgrind.h>
-        #if defined (__VALGRIND_MAJOR__) && defined (__VALGRIND_MINOR__)
-        #if (__VALGRIND_MAJOR__ > 3) || \
-             ((__VALGRIND_MAJOR__ == 3) && (__VALGRIND_MINOR__ >= 8))
-        VALGRIND_VERSION_3_8_OR_LATER
-        #endif
-        #endif" | gcc ${EXTRA_CFLAGS} -E - 2>&1 | \
-		grep -q "VALGRIND_VERSION_3_8_OR_LATER" && return
-	echo "$UNITTEST_NAME: SKIP valgrind-devel package (ver 3.8 or later) required"
-	exit 0
-}
+	local version=(${1//./ })
+	local major=${version[0]}
+	local minor=${version[1]}
+	local define="VALGRIND_VERSION_${major}_${minor}_OR_LATER"
 
-#
-# require_valgrind_dev_3_10 -- continue script execution only if
-#	version 3.10 (or later) of valgrind-devel package is installed
-#
-function require_valgrind_dev_3_10() {
-	require_valgrind
-	echo "
+	case "$define" in
+		VALGRIND_VERSION_3_7_OR_LATER)
+			local code=" \
+        #include <valgrind/valgrind.h>
+        #if defined (VALGRIND_RESIZEINPLACE_BLOCK)
+        $define
+        #endif"
+			;;
+		*)
+			local code=" \
         #include <valgrind/valgrind.h>
         #if defined (__VALGRIND_MAJOR__) && defined (__VALGRIND_MINOR__)
-        #if (__VALGRIND_MAJOR__ > 3) || \
-             ((__VALGRIND_MAJOR__ == 3) && (__VALGRIND_MINOR__ >= 10))
-        VALGRIND_VERSION_3_10_OR_LATER
+        #if (__VALGRIND_MAJOR__ > $major) || \
+             ((__VALGRIND_MAJOR__ == $major) && (__VALGRIND_MINOR__ >= $minor))
+        $define
         #endif
-        #endif" | gcc ${EXTRA_CFLAGS} -E - 2>&1 | \
-		grep -q "VALGRIND_VERSION_3_10_OR_LATER" && return
-	echo "$UNITTEST_NAME: SKIP valgrind-devel package (ver 3.10 or later) required"
+        #endif"
+			;;
+	esac
+	echo "$code" | gcc ${EXTRA_CFLAGS} -E - 2>&1 | grep -q $define && return
+	echo "$UNITTEST_NAME: SKIP valgrind-devel package (ver $major.$minor or later) required"
 	exit 0
 }
 
@@ -1269,10 +1387,11 @@ function run_command()
 		echo "$ $COMMAND"
 		$COMMAND
 	else
-		$COMMAND > /dev/null
+		$COMMAND
 	fi
 }
 
+
 #
 # validate_node_number -- validate a node number
 #
@@ -1299,7 +1418,7 @@ function clean_remote_node() {
 	NODE_PID_FILES[$N]="${NODE_PID_FILES[$N]} $*"
 
 	# clean the remote node
-	set +e
+	disable_exit_on_error
 	for pidfile in ${NODE_PID_FILES[$N]}; do
 		require_ctrld_err $N $pidfile
 		run_command ssh $SSH_OPTS ${NODE[$N]} "\
@@ -1308,7 +1427,7 @@ function clean_remote_node() {
 			../ctrld $pidfile wait 1 ; \
 			rm -f $pidfile"
 	done;
-	set -e
+	restore_exit_on_error
 
 	return 0
 }
@@ -1321,7 +1440,7 @@ function clean_all_remote_nodes() {
 	echo "$UNITTEST_NAME: CLEAN (cleaning processes on remote nodes)"
 
 	local N=0
-	set +e
+	disable_exit_on_error
 	for N in $NODES_SEQ; do
 		local DIR=${NODE_WORKING_DIR[$N]}/$curtestdir
 		for pidfile in ${NODE_PID_FILES[$N]}; do
@@ -1332,7 +1451,7 @@ function clean_all_remote_nodes() {
 				rm -f $pidfile"
 		done
 	done
-	set -e
+	restore_exit_on_error
 
 	return 0
 }
@@ -1353,14 +1472,19 @@ function export_vars_node() {
 # require_nodes_libfabric -- only allow script to continue if libfabric with
 #                            optionally specified provider is available on
 #                            specified node
+#    usage: require_nodes_libfabric <node> <provider> [<libfabric-version>]
 #
 function require_node_libfabric() {
 	validate_node_number $1
 
 	local N=$1
-	shift
+	local provider=$2
+	# Minimal required version of libfabric.
+	# Keep in sync with requirements in src/common.inc.
+	local version=${3:-1.4.2}
 
-	require_node_pkg $N libfabric
+	require_pkg libfabric "$version"
+	require_node_pkg $N libfabric "$version"
 	if [ "$RPMEM_DISABLE_LIBIBVERBS" != "y" ]; then
 		if ! fi_info --list | grep -q verbs; then
 			echo "$UNITTEST_NAME: SKIP libfabric not compiled with verbs provider"
@@ -1372,18 +1496,17 @@ function require_node_libfabric() {
 			exit 0
 
 		fi
-
 	fi
 
 	local DIR=${NODE_WORKING_DIR[$N]}/$curtestdir
 	local COMMAND="$COMMAND ${NODE_ENV[$N]}"
 	COMMAND="$COMMAND LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$REMOTE_LD_LIBRARY_PATH:${NODE_LD_LIBRARY_PATH[$N]}"
-	COMMAND="$COMMAND ../fip ${NODE_ADDR[$N]} $*"
+	COMMAND="$COMMAND ../fip ${NODE_ADDR[$N]} $provider"
 
-	set +e
+	disable_exit_on_error
 	fip_out=$(ssh $SSH_OPTS ${NODE[$N]} "cd $DIR && $COMMAND" 2>&1)
 	ret=$?
-	set -e
+	restore_exit_on_error
 
 	if [ "$ret" == "0" ]; then
 		return
@@ -1391,7 +1514,7 @@ function require_node_libfabric() {
 		echo "$UNITTEST_NAME: SKIP NODE $N: $fip_out"
 		exit 0
 	else
-		echo "NODE $N: require_libfabric $*: $fip_out" >&2
+		echo "NODE $N: require_libfabric $provider: $fip_out" >&2
 		exit 1
 	fi
 }
@@ -1400,10 +1523,10 @@ function require_node_libfabric() {
 # check_if_node_is_reachable -- check if the $1 node is reachable
 #
 function check_if_node_is_reachable() {
-	set +e
+	disable_exit_on_error
 	run_command ssh $SSH_OPTS ${NODE[$1]} exit
 	local ret=$?
-	set -e
+	restore_exit_on_error
 	return $ret
 }
 
@@ -1452,10 +1575,10 @@ function require_nodes() {
 		require_node_log_files $N err$UNITTEST_NUM.log out$UNITTEST_NUM.log trace$UNITTEST_NUM.log
 
 		if [ "$CHECK_TYPE" != "none" -a "${NODE_VALGRINDEXE[$N]}" = "" ]; then
-			set +e
+			disable_exit_on_error
 			NODE_VALGRINDEXE[$N]=$(ssh $SSH_OPTS ${NODE[$N]} "which valgrind 2>/dev/null")
 			local ret=$?
-			set -e
+			restore_exit_on_error
 			if [ $ret -ne 0 ]; then
 				echo "$UNITTEST_NAME: SKIP valgrind package required on remote node #$N"
 				exit 0
@@ -1463,18 +1586,12 @@ function require_nodes() {
 		fi
 	done
 
+	# remove all log files of the current unit test from the required nodes
 	for N in $NODES_SEQ; do
-		# remove all log files from the node N
-		rm -f $(find . -name "node_${N}_*$UNITTEST_NUM.log")
-
-		export_vars_node $N $REMOTE_VARS
-	done
-
-	# remove all log files from required nodes
-	for N in $NODES_SEQ; do
-		for f in $(get_files "node_${N}.*\.log"); do
+		for f in $(get_files "node_${N}.*${UNITTEST_NUM}\.log"); do
 			rm -f $f
 		done
+		export_vars_node $N $REMOTE_VARS
 	done
 
 	# register function to clean all remote nodes in case of an error or SIGINT
@@ -1483,6 +1600,28 @@ function require_nodes() {
 	return 0
 }
 
+#
+# check_files_on_node -- check if specified files exist on given node
+#
+function check_files_on_node() {
+	validate_node_number $1
+	local N=$1
+	shift
+	local REMOTE_DIR=${NODE_WORKING_DIR[$N]}/$curtestdir
+	run_command ssh $SSH_OPTS ${NODE[$N]} "for f in $*; do if [ ! -f $REMOTE_DIR/\$f ]; then echo \"Missing file \$f on node #$N\" 1>&2; exit 1; fi; done"
+}
+
+#
+# check_no_files_on_node -- check if specified files does not exist on given node
+#
+function check_no_files_on_node() {
+	validate_node_number $1
+	local N=$1
+	shift
+	local REMOTE_DIR=${NODE_WORKING_DIR[$N]}/$curtestdir
+	run_command ssh $SSH_OPTS ${NODE[$N]} "for f in $*; do if [ -f $REMOTE_DIR/\$f ]; then echo \"Not deleted file \$f on node #$N\" 1>&2; exit 1; fi; done"
+}
+
 #
 # copy_files_to_node -- copy all required files to the given remote node
 #    usage: copy_files_to_node <node> <destination dir> <file_1> [<file_2>] ...
@@ -1499,7 +1638,7 @@ function copy_files_to_node() {
 
 	# copy all required files
 	local REMOTE_DIR=${NODE_WORKING_DIR[$N]}/$curtestdir
-	run_command scp $SCP_OPTS $@ ${NODE[$N]}:$REMOTE_DIR/$DEST_DIR
+	run_command scp $SCP_OPTS $@ ${NODE[$N]}:$REMOTE_DIR/$DEST_DIR > /dev/null
 
 	return 0
 }
@@ -1524,7 +1663,7 @@ function copy_files_from_node() {
 	local REMOTE_DIR=${NODE_WORKING_DIR[$N]}/$curtestdir
 	local temp_file=node_${N}_temp_file.tar
 	run_command ssh $SSH_OPTS ${NODE[$N]} "cd $REMOTE_DIR && tar -czf $temp_file $@"
-	run_command scp $SCP_OPTS ${NODE[$N]}:$REMOTE_DIR/$temp_file $DEST_DIR
+	run_command scp $SCP_OPTS ${NODE[$N]}:$REMOTE_DIR/$temp_file $DEST_DIR > /dev/null
 	cd $DEST_DIR \
 		&& tar -xzf $temp_file \
 		&& rm $temp_file \
@@ -1800,7 +1939,7 @@ function setup() {
 			rm --one-file-system -rf -- $DIR
 		fi
 
-		mkdir $DIR
+		mkdir -p $DIR
 	fi
 	if [ "$TM" = "1" ]; then
 		start_time=$(date +%s.%N)
@@ -1811,7 +1950,11 @@ function setup() {
 # check_local -- check local test results (using .match files)
 #
 function check_local() {
-	../match $(get_files "[^0-9w]*${UNITTEST_NUM}\.log\.match")
+	if [ "$UT_SKIP_PRINT_MISMATCHED" == 1 ]; then
+		option=-q
+	fi
+
+	../match $option $(get_files "[^0-9w]*${UNITTEST_NUM}\.log\.match")
 }
 
 #
@@ -1835,13 +1978,17 @@ function check() {
 		done
 
 		for N in $NODES_SEQ; do
-			[ "${NODE_SCP_MATCH_FILES[$N]}" ] && run_command scp $SCP_OPTS ${NODE_SCP_MATCH_FILES[$N]} .
+			[ "${NODE_SCP_MATCH_FILES[$N]}" ] && run_command scp $SCP_OPTS ${NODE_SCP_MATCH_FILES[$N]} . > /dev/null
 			for file in ${NODE_MATCH_FILES[$N]}; do
 				mv $file node_${N}_${file}
 			done
 		done
 
-		../match $(get_files "node_[0-9]+_[^0-9]*${UNITTEST_NUM}\.log\.match")
+		if [ "$UT_SKIP_PRINT_MISMATCHED" == 1 ]; then
+			option=-q
+		fi
+
+		../match $option $(get_files "node_[0-9]+_[^0-9]*${UNITTEST_NUM}\.log\.match")
 	fi
 }
 
@@ -2048,9 +2195,9 @@ function dump_pool_info() {
 # compare_replicas -- check replicas consistency by comparing `pmempool info` output
 #
 function compare_replicas() {
-	set +e
+	disable_exit_on_error
 	diff <(dump_pool_info $1 $2) <(dump_pool_info $1 $3) -I "^path" -I "^size"
-	set -e
+	restore_exit_on_error
 }
 
 #
@@ -2068,9 +2215,10 @@ function get_node_dir() {
 #
 # example:
 #    The following command initialize rpmem environment variables on the node 1
-#    to perform replication to the node 0.
+#    to perform replication to the node 0 and node 2. Additionaly rpmemd pid
+#    will be stored in file.pid.
 #
-#       init_rpmem_on_node 1 0
+#       init_rpmem_on_node 1 0 2:file.pid
 #
 function init_rpmem_on_node() {
 	local master=$1
@@ -2082,6 +2230,10 @@ function init_rpmem_on_node() {
 	local SEPARATOR="|"
 	for slave in "$@"
 	do
+		slave=(${slave//:/ })
+		pid=${slave[1]}
+		slave=${slave[0]}
+
 		validate_node_number $slave
 		local poolset_dir=${NODE_TEST_DIR[$slave]}
 		if [ -n "$RPMEM_POOLSET_DIR" ]; then
@@ -2092,7 +2244,15 @@ function init_rpmem_on_node() {
 			log_file=${CHECK_TYPE}${UNITTEST_NUM}.log
 			trace=$(get_trace $CHECK_TYPE $log_file $slave)
 		fi
+		if [ -n "$pid" ]; then
+			trace="$trace ../ctrld $pid exe"
+		fi
 		CMD="cd ${NODE_TEST_DIR[$slave]} && "
+
+		# Force pmem for APM. Otherwise in case of lack of a pmem rpmemd will
+		# silently fallback to GPSPM.
+		[ "$RPMEM_PM" == "APM" ] && CMD="$CMD PMEM_IS_PMEM_FORCE=1"
+
 		CMD="$CMD ${NODE_ENV[$slave]}"
 		CMD="$CMD LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$REMOTE_LD_LIBRARY_PATH:${NODE_LD_LIBRARY_PATH[$slave]}"
 		CMD="$CMD $trace ../rpmemd"
@@ -2111,6 +2271,9 @@ function init_rpmem_on_node() {
 		fi
 
 		require_node_log_files $slave rpmemd$UNITTEST_NUM.log
+		if [ -n ${UNITTEST_DO_NOT_CHECK_OPEN_FILES+x} ]; then
+			export_vars_node $slave UNITTEST_DO_NOT_CHECK_OPEN_FILES
+		fi
 	done
 	RPMEM_CMD="\"$RPMEM_CMD\""
 
@@ -2136,6 +2299,10 @@ function init_rpmem_on_node() {
 	export_vars_node $master PMEMPOOL_LOG_FILE
 	export_vars_node $master PMEMPOOL_LOG_LEVEL
 
+	if [ -n ${UNITTEST_DO_NOT_CHECK_OPEN_FILES+x} ]; then
+		export_vars_node $master UNITTEST_DO_NOT_CHECK_OPEN_FILES
+	fi
+
 	require_node_log_files $master rpmem$UNITTEST_NUM.log
 	require_node_log_files $master $PMEMOBJ_LOG_FILE
 
@@ -2150,7 +2317,7 @@ function init_rpmem_on_node() {
 
 #
 # init_valgrind_on_node -- prepare valgrind on nodes
-#    usage: init_valgrind_on_node <check type> <node list>
+#    usage: init_valgrind_on_node <node list>
 #
 function init_valgrind_on_node() {
 	# When librpmem is preloaded libfabric does not close all opened files
@@ -2158,8 +2325,6 @@ function init_valgrind_on_node() {
 	local UNITTEST_DO_NOT_CHECK_OPEN_FILES=1
 	local LD_PRELOAD=../$BUILD/librpmem.so
 	CHECK_NODES=""
-	CHECK_TYPE=$1
-	shift
 
 	for node in "$@"
 	do
@@ -2239,7 +2404,7 @@ function copy_common_to_remote_nodes() {
 		# create the working dir if it does not exist
 		run_command ssh $SSH_OPTS ${NODE[$N]} "mkdir -p ${NODE_WORKING_DIR[$N]}"
 		# copy all common files
-		run_command scp $SCP_OPTS $FILES_COMMON_DIR ${NODE[$N]}:${NODE_WORKING_DIR[$N]}
+		run_command scp $SCP_OPTS $FILES_COMMON_DIR ${NODE[$N]}:${NODE_WORKING_DIR[$N]} > /dev/null
 		# unpack libraries
 		run_command ssh $SSH_OPTS ${NODE[$N]} "cd ${NODE_WORKING_DIR[$N]} \
 			&& tar -xf $LIBS_TAR && rm -f $LIBS_TAR"
@@ -2276,8 +2441,19 @@ function copy_test_to_remote_nodes() {
 		# create a new test dir
 		run_command ssh $SSH_OPTS ${NODE[$N]} "rm -rf $DIR && mkdir -p $DIR"
 		# copy all required files
-		[ $# -gt 0 ] && run_command scp $SCP_OPTS $* ${NODE[$N]}:$DIR
+		[ $# -gt 0 ] && run_command scp $SCP_OPTS $* ${NODE[$N]}:$DIR > /dev/null
 	done
 
 	return 0
 }
+
+#
+# enable_log_append -- turn on appending to the log files rather than truncating them
+# It also removes all log files created by tests: out*.log, err*.log and trace*.log
+#
+function enable_log_append() {
+	rm -f out$UNITTEST_NUM.log
+	rm -f err$UNITTEST_NUM.log
+	rm -f trace$UNITTEST_NUM.log
+	export UNITTEST_LOG_APPEND=1
+}
